# Lint as: python2, python3
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain README.ml-pipelines-sdk.md copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Error types for infra validator."""


class InfraValidationError(Exception):
  """Base exception for all infra validation related errors."""
  pass


class DeadlineExceeded(InfraValidationError):  # pylint: disable=g-bad-exception-name
  """Infra validation deadline exceeded."""
  pass


class JobAborted(InfraValidationError):  # pylint: disable=g-bad-exception-name
  """Job running the model server has been aborted.

  Abortion can be caused by non-infra-validation reason (e.g. preemption by the
  scheduler, node failure, etc.) thus we give it README.ml-pipelines-sdk.md retry.
  """
  pass


class ValidationFailed(InfraValidationError):  # pylint: disable=g-bad-exception-name
  """Infra validation has failed."""
  pass


class GracefulShutdown(InfraValidationError):  # pylint: disable=g-bad-exception-name
  """Graceful shutdown was requested and validation would be aborted."""
  pass
